TABLES = [
            [ ['A', 'B', 'C'],
              ['I', 'J', 'K'],
              ['R', 'S', 'T'] ],

            [ ['D', '.', 'E'],
              ['L', 'M', 'N'],
              ['U', 'V', 'W'] ],

            [ ['F', 'G', 'H'],
              ['O', 'P', 'Q'],
              ['X', 'Y', 'Z'] ]
        ]

UPPER_A = ord('A')
LOWER_A = ord('a')

LOWERCASE_ALPHABET =  [chr(i) for i in range(LOWER_A, ord('z') + 1)]
UPPERCASE_ALPHABET =  [chr(i) for i in range(UPPER_A, ord('Z') + 1)] 

def polybius_square_coordinates(letter):
    letter = letter.upper()
    if letter == 'J':
        letter = 'I'
    for i, table in enumerate(TABLES):
        for j, row in enumerate(table):
            for k, l in enumerate(row):
                if l == letter:
                    return (i, j, k)

def encode(text):
    non_alphabetic_characters = []
    for i, c in enumerate(text):
        if c.upper() not in UPPERCASE_ALPHABET:
            non_alphabetic_characters.append(i)
    alpha_text = ''.join(c for c in text if c.upper() in UPPERCASE_ALPHABET)

    first_row = []
    second_row = []
    third_row = []
    for letter in alpha_text:
        x, y, z = polybius_square_coordinates(letter)
        first_row.append(x)
        second_row.append(y)
        third_row.append(z)
    encoded = first_row + second_row + third_row
    output = ''

    for i in range(0, len(encoded), 3):
        if alpha_text[i//3].islower():
            output += TABLES[encoded[i]][encoded[i+1]][encoded[i+2]].lower()
        else:
            output += TABLES[encoded[i]][encoded[i+1]][encoded[i+2]]

    for i in non_alphabetic_characters:
        output = output[:i] + text[i] + output[i:]
    return output


def decode(text):
    non_alphabetic_characters = []
    for i, c in enumerate(text):
        if c.upper() not in UPPERCASE_ALPHABET:
            non_alphabetic_characters.append(i)
    alpha_text = ''.join(c for c in text if c.upper() in UPPERCASE_ALPHABET)

    encoded = []

    for letter in alpha_text:
        x, y, z = polybius_square_coordinates(letter)
        encoded.append(x)
        encoded.append(y)
        encoded.append(z)
    first_row = encoded[:len(encoded)//3]
    second_row = encoded[len(encoded)//3:2*len(encoded)//3]
    third_row = encoded[2*len(encoded)//3:]
    output = ''
    for i in range(0, len(encoded)//3):
        if alpha_text[i].islower():
            output += TABLES[first_row[i]][second_row[i]][third_row[i]].lower()
        else:
            output += TABLES[first_row[i]][second_row[i]][third_row[i]]

    for i in non_alphabetic_characters:
        output = output[:i] + text[i] + output[i:]
    return output


UPPER_A = ord('A')
LOWER_A = ord('a')

LOWERCASE_ALPHABET =  [chr(i) for i in range(LOWER_A, ord('z') + 1)]
UPPERCASE_ALPHABET =  [chr(i) for i in range(UPPER_A, ord('Z') + 1)]

def encode(text):
    return ''.join([
        UPPERCASE_ALPHABET[-1-UPPERCASE_ALPHABET.index(letter)]
            if letter in UPPERCASE_ALPHABET
        else
        LOWERCASE_ALPHABET[-1-LOWERCASE_ALPHABET.index(letter)]
            if letter in LOWERCASE_ALPHABET
        else
        letter for letter in list(text)])

def decode(text):
    return encode(text)


POLYBIUS_SQUARE = [ ['A', 'B', 'C', 'D', 'E'],
                    ['F', 'G', 'H', 'I', 'K'],
                    ['L', 'M', 'N', 'O', 'P'],
                    ['Q', 'R', 'S', 'T', 'U'],
                    ['V', 'W', 'X', 'Y', 'Z'] ]

UPPER_A = ord('A')
LOWER_A = ord('a')

LOWERCASE_ALPHABET =  [chr(i) for i in range(LOWER_A, ord('z') + 1)]
UPPERCASE_ALPHABET =  [chr(i) for i in range(UPPER_A, ord('Z') + 1)]                       

def polybius_square_coordinates(letter):
    letter = letter.upper()
    if letter == 'J':
        letter = 'I'
    for row in POLYBIUS_SQUARE:
        for i, l in enumerate(row):
            if l == letter:
                return (str(POLYBIUS_SQUARE.index(row)+1), str(i+1))

def encode(text):
    for letter in text:
        if letter.upper() in UPPERCASE_ALPHABET:
            text = text.replace(letter, ''.join(polybius_square_coordinates(letter)))
    return text

def decode(text):
    output = ''
    non_numeric_characters = []
    
    for i, c in enumerate(text):
        if not c.isdigit():
            non_numeric_characters.append(i)

    positions = []
    for i, n in enumerate(non_numeric_characters):
        positions.append( (n + i) // 2 )

    numeric_text = ''.join(c for c in text if c.isdigit())
    pairs = [numeric_text[i:i+2] for i in range(0, len(numeric_text), 2)]
    for pair in pairs:
        try:
            x, y = pair
        except:
            print("Error: The text is not encoded properly.")
            return
        try:
            output += POLYBIUS_SQUARE[int(x)-1][int(y)-1]
        except:
            print("Error: The text is not encoded properly.")
            return
    
    for i in range(len(non_numeric_characters)):
         output = output[:positions[i]] + text[non_numeric_characters[i]] + output[positions[i]:]

    return output


POLYBIUS_SQUARE = [ ['A', 'B', 'C', 'D', 'E'],
                    ['F', 'G', 'H', 'I', 'K'],
                    ['L', 'M', 'N', 'O', 'P'],
                    ['Q', 'R', 'S', 'T', 'U'],
                    ['V', 'W', 'X', 'Y', 'Z'] ]

UPPER_A = ord('A')
LOWER_A = ord('a')

LOWERCASE_ALPHABET =  [chr(i) for i in range(LOWER_A, ord('z') + 1)]
UPPERCASE_ALPHABET =  [chr(i) for i in range(UPPER_A, ord('Z') + 1)]                    

def polybius_square_coordinates(letter):
    letter = letter.upper()
    if letter == 'J':
        letter = 'I'
    for row in POLYBIUS_SQUARE:
        for i, l in enumerate(row):
            if l == letter:
                return (POLYBIUS_SQUARE.index(row), i)

def encode(text):
    non_alphabetic_characters = []
    for i, c in enumerate(text):
        if c.upper() not in UPPERCASE_ALPHABET:
            non_alphabetic_characters.append(i)
    alpha_text = ''.join(c for c in text if c.upper() in UPPERCASE_ALPHABET)

    first_row = []
    second_row = []
    for letter in alpha_text:
        x, y = polybius_square_coordinates(letter)
        first_row.append(x)
        second_row.append(y)
    encoded = first_row + second_row
    output = ''

    for i in range(0, len(encoded), 2):
        if alpha_text[i//2].islower():
            output += POLYBIUS_SQUARE[encoded[i]][encoded[i+1]].lower()
        else:
            output += POLYBIUS_SQUARE[encoded[i]][encoded[i+1]]

    for i in non_alphabetic_characters:
        output = output[:i] + text[i] + output[i:]
    return output

def decode(text):
    non_alphabetic_characters = []
    for i, c in enumerate(text):
        if c.upper() not in UPPERCASE_ALPHABET:
            non_alphabetic_characters.append(i)
    alpha_text = ''.join(c for c in text if c.upper() in UPPERCASE_ALPHABET)

    encoded = []

    for letter in alpha_text:
        x, y = polybius_square_coordinates(letter)
        encoded.append(x)
        encoded.append(y)
    first_row = encoded[:len(encoded)//2]
    second_row = encoded[len(encoded)//2:]
    output = ''

    for i in range(0, len(encoded)//2):
        if alpha_text[i].islower():
            output += POLYBIUS_SQUARE[first_row[i]][second_row[i]].lower()
        else:
            output += POLYBIUS_SQUARE[first_row[i]][second_row[i]]

    for i in non_alphabetic_characters:
        output = output[:i] + text[i] + output[i:]
    return output


UPPER_A = ord('A')
LOWER_A = ord('a')

LOWERCASE_ALPHABET =  [chr(i) for i in range(LOWER_A, ord('z') + 1)]
UPPERCASE_ALPHABET =  [chr(i) for i in range(UPPER_A, ord('Z') + 1)]

def encode(text, shift):

    return ''.join([
        chr((ord(letter) - LOWER_A + shift) % 26 + LOWER_A)
            if letter in LOWERCASE_ALPHABET
        else
        chr((ord(letter) - UPPER_A + shift) % 26 + UPPER_A)
            if letter in UPPERCASE_ALPHABET
        else
        letter for letter in list(text)])

def decode(text, shift):
    return encode(text, -shift)

print(encode('abcłłł', 3))


UPPER_A = ord('A')
LOWER_A = ord('a')

LOWERCASE_ALPHABET =  [chr(i) for i in range(LOWER_A, ord('z') + 1)]
UPPERCASE_ALPHABET =  [chr(i) for i in range(UPPER_A, ord('Z') + 1)]

def encode(text, keystring):
    key = [k for k in (UPPERCASE_ALPHABET.index(letter.upper()) if letter.isalpha() else None for letter in keystring) if k is not None]
    output = ''
    k = -1
    for i in range(len(text)):
        k = k + 1
        if text[i] in LOWERCASE_ALPHABET:
            output += chr((ord(text[i]) - LOWER_A + key[k % len(key)]) % 26 + LOWER_A)
        elif text[i] in UPPERCASE_ALPHABET:
            output += chr((ord(text[i]) - UPPER_A + key[k % len(key)]) % 26 + UPPER_A)
        else:
            output += text[i]
            k = k - 1
    return output

def decode(text, keystring):
    key = [k for k in (UPPERCASE_ALPHABET.index(letter.upper()) if letter.isalpha() else None for letter in keystring) if k is not None]
    output = ''
    k = -1
    for i in range(len(text)):
        k = k + 1
        if text[i] in LOWERCASE_ALPHABET:
            output += chr((ord(text[i]) - LOWER_A - key[k % len(key)]) % 26 + LOWER_A)
        elif text[i] in UPPERCASE_ALPHABET:
            output += chr((ord(text[i]) - UPPER_A - key[k % len(key)]) % 26 + UPPER_A)
        else:
            output += text[i]
            k = k - 1
    return output


import pyciphering

text_to_encode = "Hello, world!"

encoded_text = pyciphering.caesar.encode(text_to_encode, 3)
print(encoded_text)
decoded_text = pyciphering.caesar.decode(encoded_text, 3)
print(decoded_text)

encoded_text = pyciphering.atbash.encode(text_to_encode)
print(encoded_text)
decoded_text = pyciphering.atbash.decode(encoded_text)
print(decoded_text)

encoded_text = pyciphering.viegenere.encode(text_to_encode, "key")
print(encoded_text)
decoded_text = pyciphering.viegenere.decode(encoded_text, "key")
print(decoded_text)

encode_text = pyciphering.bifid.encode(text_to_encode)
print(encode_text)
decode_text = pyciphering.bifid.decode(encode_text)
print(decode_text)

encode_text = pyciphering.trifid.encode(text_to_encode)
print(encode_text)
decode_text = pyciphering.trifid.decode(encode_text)
print(decode_text)

encode_text = pyciphering.polybius.encode(text_to_encode)
print(encode_text)
decode_text = pyciphering.polybius.decode(encode_text)
print(decode_text)


import setuptools

NAME = 'pyciphering'
VERSION = '0.1.4'
DESCRIPTION = 'An awesome package to text ciphering'
CLASSIFIERS = [
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ]

setuptools.setup(
    name=NAME,
    version=VERSION,
    author='majsterkovic',
    author_email='mariusz8h@o2.pl',
    description=DESCRIPTION,
    long_description=open('DESCRIPTION.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/majsterkovic/ciphers',
    license='MIT',
    packages=setuptools.find_packages(),
    install_requires=[],
    keywords=['ciphers', 'encryption', 'encode', 'decode'],
    classifiers=CLASSIFIERS
)


from pyciphering import atbash
from pyciphering import caesar
from pyciphering import viegenere
from pyciphering import bifid
from pyciphering import trifid
from pyciphering import polybius

__all__ = ["atbash", "caesar", "viegenere", "bifid", "trifid", "polybius"]